/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: filters.c,v 1.4 1999/04/12 18:38:44 bazsi Exp $
 *
 ***************************************************************************/

#include "filters.h"
#include "format.h"
#include "syslog-names.h"
#include <syslog.h>

#define CLASS_DEFINE
#include "filters.h.x"
#undef CLASS_DEFINE

#include "filters.c.x"

struct log_filter *make_filter_rule(const char *name, struct filter_expr_node *root)
{
	NEW(log_filter, self);

	self->name = c_format("%z", name);
	self->root = root;
	return self;
}

/* CLASS:
   (class
     (name filter_expr_op)
     (super filter_expr_node)
     (vars
       (left object filter_expr_node)
       (right object filter_expr_node)))
*/

static int do_filter_and(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_op, self, c);

	return FILTER_RULE_EVAL(self->left, log) && FILTER_RULE_EVAL(self->right, log);
}

struct filter_expr_node *make_filter_and(struct filter_expr_node *l, struct filter_expr_node *r)
{
	NEW(filter_expr_op, self);
       
	self->super.eval = do_filter_and;
	self->left = l;
	self->right = r;
	return &self->super;
}

static int do_filter_or(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_op, self, c);

	return FILTER_RULE_EVAL(self->left, log) || FILTER_RULE_EVAL(self->right, log);
}
 
struct filter_expr_node *make_filter_or(struct filter_expr_node *l, struct filter_expr_node *r)
{
	NEW(filter_expr_op, self);

	self->super.eval = do_filter_or;
	self->left = l;
	self->right = r;
	return &self->super;
}

/* CLASS:
   (class
     (name filter_expr_pri)
     (super filter_expr_node)
     (vars
       (mask simple UINT32)
       (valid simple UINT32)
       (values pointer (simple "struct sl_name"))
       (cmp simple int)))
*/

static int do_filter_pri(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_pri, self, c);
	UINT32 pri = log->pri & self->mask;
	int i;
	UINT32 bits = self->valid;

	for (i = 0; bits && self->values[i].name; i++, bits >>= 1) {
		if ((bits & 1) && self->values[i].value == pri) {
			return !self->super.comp;
		}
	}
	return self->super.comp;
}

struct filter_expr_node *make_filter_facility(UINT32 valid)
{
	NEW(filter_expr_pri, self);

	self->super.eval = do_filter_pri;
	self->mask = LOG_FACMASK;
	self->values = sl_facilities;
	self->valid = valid;
	return &self->super;
}

struct filter_expr_node *make_filter_level(UINT32 valid)
{
	NEW(filter_expr_pri, self);

	self->super.eval = do_filter_pri;
	self->mask = LOG_PRIMASK;
	self->values = sl_levels;
	self->valid = valid;
	return &self->super;
}

/* CLASS:
   (class
     (name filter_expr_prog)
     (super filter_expr_node)
     (vars
       (name string)))
*/

int do_filter_prog(struct filter_expr_node *c, struct log_info *log)
{
	CAST(filter_expr_prog, self, c);

	return (log->program && (self->name->length == log->program->length) && (memcmp(self->name->data, log->program->data, self->name->length) == 0)) ^ c->comp;
}

struct filter_expr_node *make_filter_prog(const char *name)
{
	NEW(filter_expr_prog, self);

	self->super.eval = do_filter_prog;
	self->name = c_format("%z", name);
	return &self->super;
}
